use xml_builder::{XMLBuilder, XMLElement, XMLVersion};

use crate::{xml_tag_attribute, xml_tag_text};

pub fn generate(
    config: crate::config::Syndication,
    post_index: &Vec<crate::index::BlogPost>,
) -> String {
    let mut xml = XMLBuilder::new()
        .version(XMLVersion::XML1_0)
        .encoding("UTF-8".into())
        .build();

    let mut feed = XMLElement::new("feed");
    feed.add_attribute("xmlns", "http://www.w3.org/2005/Atom");

    xml_tag_text!("id", config.link.clone(), feed);

    xml_tag_text!("title", config.title, feed);

    xml_tag_attribute!("link", feed, ("href", &config.link));

    let mut author = XMLElement::new("author");
    let mut name = XMLElement::new("name");
    name.add_text(config.author).unwrap();
    author.add_child(name).unwrap();
    feed.add_child(author).unwrap();

    let last_update = chrono::DateTime::from_timestamp(last_update(post_index), 0).unwrap();
    xml_tag_text!("updated", last_update.to_rfc3339(), feed);

    if config.subtitle.is_some() {
        xml_tag_text!("subtitle", config.subtitle.unwrap(), feed);
    }

    if config.icon.is_some() {
        xml_tag_text!("icon", config.icon.unwrap(), feed);
    }

    for post in post_index {
        let mut entry = XMLElement::new("entry");

        xml_tag_text!("title", post.title.clone(), entry);

        xml_tag_attribute!(
            "link",
            entry,
            (
                "href",
                &format!(
                    "{}{}/{}",
                    &config.link, &config.blog_root, &post.output_file_name
                )
            )
        );

        xml_tag_text!(
            "id",
            format!(
                "{}{}/{}",
                &config.link, &config.blog_root, &post.output_file_name
            ),
            entry
        );
        let last_update = chrono::DateTime::from_timestamp(post.last_updated, 0)
            .unwrap()
            .to_rfc3339();
        xml_tag_text!("updated", last_update, entry);

        let mut content = XMLElement::new("content");
        content.add_attribute("type", "html");
        content
            .add_text(crate::syndication::escape_html(post.content.clone()))
            .unwrap();
        entry.add_child(content).unwrap();

        feed.add_child(entry).unwrap();
    }

    xml.set_root_element(feed);
    let mut writer: Vec<u8> = Vec::new();
    xml.generate(&mut writer).unwrap();

    let mut output_xml = String::new();

    for e in writer {
        output_xml += &e.as_ascii().unwrap().to_string();
    }

    output_xml
}

fn last_update(post_index: &Vec<crate::index::BlogPost>) -> i64 {
    let mut last_timestamp: i64 = i64::MIN;

    for post in post_index {
        if post.sort_date > last_timestamp {
            last_timestamp = post.sort_date;
        }
    }
    last_timestamp
}
